/**
 * Define api of apeman-api-recover
 * @function define
 * @param {Object} [options] - Optional settings
 * @param {boolean} [options.logging] - Enable logger
 * @returns {Object} - API configuration
 */
'use strict';

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var co = require('co');
var assert = require('assert');
var uuid = require('uuid');
var defaults = require('defaults');
var aslogger = require('aslogger');
var apemanApiSign = require('apeman-api-sign');
var describe = require('./describe');
var load = require('apeman-api-client/load');
var apemanerror = require('apemanerror');
var _apemanerror$ErrorCod = apemanerror.ErrorCodes;
var RESOURCE_NOT_FOUND_ERROR = _apemanerror$ErrorCod.RESOURCE_NOT_FOUND_ERROR;
var TOKEN_WRONG_ERROR = _apemanerror$ErrorCod.TOKEN_WRONG_ERROR;
var TOKEN_EXPIRED_ERROR = _apemanerror$ErrorCod.TOKEN_EXPIRED_ERROR;

/** @lends define */

function define(User, UserSign, UserRecover) {
  var options = arguments.length <= 3 || arguments[3] === undefined ? {} : arguments[3];

  assert.ok(UserRecover, 'UserRecover is required.');
  assert.ok(!!(UserRecover.$$inherited && UserRecover.$$inherited.ApUserRecover), 'UserRecover should inherit ApUserRecover.');
  var passwordSpec = options.passwordSpec;
  var logging = options.logging;

  var logger = aslogger({ disabled: !logging });

  var signApi = apemanApiSign(User, UserSign, {
    passwordSpec: passwordSpec, logging: logging
  });

  return {
    ping: function ping() {
      var pong = arguments.length <= 0 || arguments[0] === undefined ? 'pong' : arguments[0];

      return co(regeneratorRuntime.mark(function _callee() {
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                logger.trace('pong:', pong);
                return _context.abrupt('return', pong);

              case 2:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));
    },
    issue: function issue(email) {
      var _defaults = defaults(options, {
        expireDuration: 26 * 60 * 60 * 10000
      });

      var expireDuration = _defaults.expireDuration;

      return co(regeneratorRuntime.mark(function _callee2() {
        var user, token, userId, recover;
        return regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                _context2.t0 = email;

                if (!_context2.t0) {
                  _context2.next = 5;
                  break;
                }

                _context2.next = 4;
                return User.findOne({ where: { email: email } });

              case 4:
                _context2.t0 = _context2.sent;

              case 5:
                user = _context2.t0;

                if (user) {
                  _context2.next = 8;
                  break;
                }

                throw apemanerror(RESOURCE_NOT_FOUND_ERROR);

              case 8:
                token = uuid.v4().replace(/\-/g, '');
                userId = user.id;
                _context2.next = 12;
                return UserRecover.findOrCreate({
                  where: { userId: userId },
                  defaults: { userId: userId, token: token }
                });

              case 12:
                recover = _context2.sent[0];
                _context2.next = 15;
                return recover.update({
                  token: token,
                  expireAt: new Date(new Date().getTime() + expireDuration)
                });

              case 15:
                return _context2.abrupt('return', { token: token, expireDuration: expireDuration, userId: userId });

              case 16:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));
    },
    check: function check(token) {
      var s = this;
      return co(regeneratorRuntime.mark(function _callee3() {
        var recover;
        return regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                _context3.next = 2;
                return s._getRecover(token);

              case 2:
                recover = _context3.sent;
                return _context3.abrupt('return', !!recover);

              case 4:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));
    },
    update: function update(token, newPassword) {
      var s = this;
      var session = s.session;

      return co(regeneratorRuntime.mark(function _callee4() {
        var sign, recover, userId, userSigns, userSign, keys;
        return regeneratorRuntime.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                _context4.next = 2;
                return load(signApi, { $$ctx: s });

              case 2:
                sign = _context4.sent;
                _context4.next = 5;
                return s._getRecover(token);

              case 5:
                recover = _context4.sent;
                userId = recover.userId;
                _context4.next = 9;
                return UserSign.findAll({
                  where: { userId: userId },
                  include: [UserSign.getRef('user')]
                });

              case 9:
                userSigns = _context4.sent;
                userSign = userSigns[0];

                session.signed = apemanApiSign.formatSigned(userSign.user, userSign);
                keys = userSigns.reduce(function (keys, userSign) {
                  return Object.assign(keys, _defineProperty({}, userSign.kind, userSign.key));
                }, {});
                _context4.next = 15;
                return sign.signmod(keys, newPassword);

              case 15:
                _context4.next = 17;
                return sign.signout();

              case 17:
                _context4.next = 19;
                return recover.update({
                  doneAt: new Date()
                });

              case 19:
                return _context4.abrupt('return', { key: userSign.key, password: newPassword });

              case 20:
              case 'end':
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));
    },


    /**
     * Get recover from token
     * @param {string} token - Token to search
     * @throws {Error} - Unless valid token found
     * @returns {Promise}
     */
    _getRecover: function _getRecover(token) {
      return co(regeneratorRuntime.mark(function _callee5() {
        var recover;
        return regeneratorRuntime.wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                _context5.t0 = token;

                if (!_context5.t0) {
                  _context5.next = 5;
                  break;
                }

                _context5.next = 4;
                return UserRecover.findOne({ token: token });

              case 4:
                _context5.t0 = _context5.sent;

              case 5:
                recover = _context5.t0;

                if (recover) {
                  _context5.next = 8;
                  break;
                }

                throw apemanerror(TOKEN_WRONG_ERROR);

              case 8:
                if (!recover.expired) {
                  _context5.next = 10;
                  break;
                }

                throw apemanerror(TOKEN_EXPIRED_ERROR);

              case 10:
                if (!recover.done) {
                  _context5.next = 12;
                  break;
                }

                throw apemanerror(TOKEN_EXPIRED_ERROR);

              case 12:
                return _context5.abrupt('return', recover);

              case 13:
              case 'end':
                return _context5.stop();
            }
          }
        }, _callee5, this);
      }));
    },


    /**
     * A hook which runs before method invoked
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @returns {Promise}
     */
    $before: function $before(methodName, params) {
      var s = this;
      var state = s.state;

      return co(regeneratorRuntime.mark(function _callee6() {
        return regeneratorRuntime.wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                state.invokedAt = new Date();
                logger.info(methodName + ' started...');

              case 2:
              case 'end':
                return _context6.stop();
            }
          }
        }, _callee6, this);
      }));
    },


    /**
     * A hook which runs after method invoked
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @param {?*} returns - Return values
     * @returns {Promise}
     */
    $after: function $after(methodName, params, returns) {
      var s = this;
      var state = s.state;

      return co(regeneratorRuntime.mark(function _callee7() {
        var took;
        return regeneratorRuntime.wrap(function _callee7$(_context7) {
          while (1) {
            switch (_context7.prev = _context7.next) {
              case 0:
                took = new Date() - state.invokedAt;

                logger.info('...' + methodName + ' done! (' + took + 'ms)');

              case 2:
              case 'end':
                return _context7.stop();
            }
          }
        }, _callee7, this);
      }));
    },

    /**
     * Method specifications
     */
    $spec: describe()

  };
}

module.exports = define;
//# sourceMappingURL=data:application/json;base64,bnVsbA==